==> archibald-7.x-2.0.tar.gz: archibald/includes/dsb_api.class.php <==
<?php

/**
 * @file
 * class that provides the connection to the central catalog of archibald
 */

/**
 * This is the parent class for all Publish-Classes.
 * Specially for
 *
 *  - ArchibaldLomPublish
 *  - CommentsOM_Pub_Publish
 *  - ArchibaldRatingsPublish
 *
 * @author cmueller <c.mueller@educa.ch>
 */
abstract class ArchibaldAbstractDsbApi {

  /**
   * array returned from converted to object
   * @see archibald_load_content_partner()
   * @var object
   */
  protected $contentPartner = NULL;

  /**
   * current processed object
   * @var mixed
   */
  protected $objectToPublish = NULL;

  /**
   * auth the client at the publich lom server
   *
   * @param $auth_token string
   *   called by reference
   *   current auth token
   *
   * @param $force boolean
   *   default FALSE
   *   TRUE: force refresh
   *
   * @return boolean
   *   return auth object on success, else FALSE
   */
  protected function lomApiAuth(&$auth_token = NULL, $force = FALSE) {
    $auth_token = variable_get('archibald_auth_key_' . $this->getContentPartnerCredential()->username);
    $expire = variable_get('archibald_auth_expire' . $this->getContentPartnerCredential()->username);

    if ($force == TRUE) {
      $expire = 0;
    }

    if (empty($auth_token) || time() >= $expire) {
      $client_private_key = $this->getClientPrivateKeyFile();
      $username           = $this->getContentPartnerCredential()->username;
      $sec_data           = $username . time();
      openssl_sign($sec_data, $signature, $client_private_key);
      openssl_free_key($client_private_key);

      $postarr = array(
        'user' => $username,
        'signature' => base64_encode($signature),
        'sec_data' => $sec_data,
      );

      $result_raw = $this->openUrl(variable_get('archibald_api_url', 'https://main-api-dsb.educa.ch') . '/auth', $postarr, TRUE);
      $auth = json_decode($result_raw);

      if (empty($auth)) {
        watchdog('lom_auth', 'Auth error. Server throws "!raw_message", contact your software distributor.',
          array(
            '!raw_message' => $result_raw,
          ),
          WATCHDOG_ERROR
        );

        if (function_exists('drush_log')) {
          drush_log('Auth: ' . $result_raw, 'error');
        }

        return FALSE;
      }

      if (empty($auth->data->AuthToken)) {
        watchdog('lom_auth', 'Auth error. Server throws "@api_return", contact your software distributor.',
          array(
            '!raw_message' => $result_raw,
            '@api_return' => print_r($auth, TRUE),
          ),
          WATCHDOG_ERROR
        );

        if (function_exists('drush_log')) {
          drush_log('Auth: ' . print_r($auth, TRUE), 'error');
        }

        return FALSE;
      }

      $auth_token = $auth->data->AuthToken;

      variable_set('archibald_auth_key_' . $this->getContentPartnerCredential()->username, $auth->data->AuthToken);
      variable_set('archibald_auth_expire_' . $this->getContentPartnerCredential()->username, $auth->data->Expire);

      if (function_exists('drush_log')) {
        drush_log('Auth: ' . $auth->code . ' (' . $auth->message . ')', $auth->code == 200 ? 'ok' : 'error');
      }
    }
    return !empty($auth);
  }

  /**
   * Open an url on https using curl and return content

   *
   * @param string $url
   *   the url to open
   * @param array $post
   *   the post variables
   * @param bool $ssl
   *   if TRUE, use ssl
   * @param string $refer
   *   referer (optional)
   * @param mixed $usecookie
   *   if TRUE, cookie.txt will be used as default, or the usecookie value.
   * @param mixed $progress_function
   *   callback function with progress informations
   *
   * @return string
   */
  public function openUrl($url, $post = array(), $ssl = FALSE, $refer = '', $usecookie = FALSE, $progress_function = '') {

    if ($usecookie) {
      if (file_exists($usecookie)) {
        if (!is_writable($usecookie)) {
          return 'Can\'t write to ' . $usecookie . ' cookie file, ' . 'change file permission to 777 or remove read only for windows.';
        }
      }
      else {
        $usecookie = 'cookie.txt';
        if (!is_writable($usecookie)) {
          return 'Can\'t write to $usecookie cookie file, ' . 'change file permission to 777 or remove read only for windows.';
        }
      }
    }
    $ch = curl_init();

    curl_setopt($ch, CURLOPT_URL, $url);

    if ($ssl == TRUE) {
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
    }

    if (!empty($post)) {
      curl_setopt($ch, CURLOPT_POST, TRUE);
      curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
    }

    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Archibald/2.0 (@' . @$_SERVER['HTTP_HOST'] . ')');
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));

    if (!empty($progress_function) && defined('CURLOPT_PROGRESSFUNCTION')) {
      curl_setopt($ch, CURLOPT_NOPROGRESS, FALSE);
      curl_setopt($ch, CURLOPT_PROGRESSFUNCTION, $progress_function);
    }

    if (!empty($usecookie)) {
      curl_setopt($ch, CURLOPT_COOKIEJAR, $usecookie);
      curl_setopt($ch, CURLOPT_COOKIEFILE, $usecookie);
    }

    if (!empty($refer)) {
      curl_setopt($ch, CURLOPT_REFERER, $refer);
    }
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

    $result = curl_exec($ch);
    if (empty($result)) {
      $result = curl_exec($ch);
    }
    curl_close($ch);
    return $result;
  }

  /**
   * get open ssl ressource for private key
   *
   * @return resource
   */
  protected function getClientPrivateKeyFile() {
    $content_partner = $this->getContentPartnerCredential();

    if (empty($content_partner)) {
      return FALSE;
    }

    $private_key_file = $content_partner->key_file;
    return openssl_pkey_get_private(file_get_contents($private_key_file->uri), $content_partner->password);
  }

  /**
   * check if current content partner can access to central catalog
   *
   * @return bollean
   */
  public function contentPartnerCanCentral() {
    return ($this->getClientPrivateKeyFile() === FALSE) ? FALSE : TRUE;
  }

  /**
   * get api login credentials
   *
   * @return object
   *   username
   *   password
   *   key_fid
   *   key_file
   *   icon_fid
   *   icon_file
   *   file_server_type
   */
  protected function getContentPartnerCredential() {
    if (!empty($this->objectToPublish->lom_stats->content_partner)) {
      return (object) $this->objectToPublish->lom_stats->content_partner;
    }
    if (!empty($this->contentPartner)) {
      return $this->contentPartner;
    }

    return NULL;
  }

  /**
   * init content_partner by lom_id
   *
   * @param string $lom_id
   */
  public function loadContentPartnerByLomId($lom_id) {

    $stats = db_query('SELECT content_partner_id FROM {archibald_lom_stats} WHERE lom_id = :lom_id', array(':lom_id' => $lom_id));
    $stats_values = $stats->fetchAssoc();
    $this->contentPartner = (object) archibald_load_content_partner($stats_values['content_partner_id']);
  }

  /**
   * init content_partner by content_partner_id
   *
   * @param integer $content_partner_id
   */
  public function loadContentPartnerByContentPartnerId($content_partner_id) {
    $this->contentPartner = (object) archibald_load_content_partner($content_partner_id);
  }

  /**
   * Each sub class must implement this method.
   */
  public abstract function cron();
}

==> archibald-7.x-2.1.tar.gz: archibald/includes/dsb_api.class.php <==
<?php

/**
 * @file
 * class that provides the connection to the central catalog of archibald
 */

/**
 * This is the parent class for all Publish-Classes.
 * Specially for
 *
 *  - ArchibaldLomPublish
 *  - CommentsOM_Pub_Publish
 *  - ArchibaldRatingsPublish
 *
 * @author cmueller <c.mueller@educa.ch>
 */
abstract class ArchibaldAbstractDsbApi {

  /**
   * array returned from converted to object
   * @see archibald_load_content_partner()
   * @var object
   */
  protected $contentPartner = NULL;

  /**
   * current processed object
   * @var mixed
   */
  protected $objectToPublish = NULL;

  /**
   * auth the client at the publich lom server
   *
   * @param $auth_token string
   *   called by reference
   *   current auth token
   *
   * @param $force boolean
   *   default FALSE
   *   TRUE: force refresh
   *
   * @return boolean
   *   return auth object on success, else FALSE
   */
  protected function lomApiAuth(&$auth_token = NULL, $force = FALSE) {
    $auth_token = variable_get('archibald_auth_key_' . $this->getContentPartnerCredential()->username);
    $expire = variable_get('archibald_auth_expire' . $this->getContentPartnerCredential()->username);

    if ($force == TRUE) {
      $expire = 0;
    }

    if (empty($auth_token) || time() >= $expire) {
      $client_private_key = $this->getClientPrivateKeyFile();
      $username           = $this->getContentPartnerCredential()->username;
      $sec_data           = $username . time();
      openssl_sign($sec_data, $signature, $client_private_key);
      openssl_free_key($client_private_key);

      $postarr = array(
        'user' => $username,
        'signature' => base64_encode($signature),
        'sec_data' => $sec_data,
      );

      $result_raw = $this->openUrl(variable_get('archibald_api_url', 'https://main-api-dsb.educa.ch') . '/auth', $postarr, TRUE);
      $auth = json_decode($result_raw);

      if (empty($auth)) {
        watchdog('lom_auth', 'While executing auth command. The server throws "@message", please contact your software distributor.',
          array(
            '@message' => $result_raw,
          ),
          WATCHDOG_ERROR
        );

        if (function_exists('drush_log')) {
          drush_log('Auth: ' . $result_raw, 'error');
        }

        return FALSE;
      }

      if (empty($auth->data->AuthToken)) {
        watchdog('lom_auth', 'While executing auth command. The server throws "@message", please contact your software distributor.',
          array(
            '@message' => print_r($auth, TRUE),
          ),
          WATCHDOG_ERROR
        );

        if (function_exists('drush_log')) {
          drush_log('Auth: ' . print_r($auth, TRUE), 'error');
        }

        return FALSE;
      }

      $auth_token = $auth->data->AuthToken;

      variable_set('archibald_auth_key_' . $this->getContentPartnerCredential()->username, $auth->data->AuthToken);
      variable_set('archibald_auth_expire_' . $this->getContentPartnerCredential()->username, $auth->data->Expire);

      if (function_exists('drush_log')) {
        drush_log('Auth: ' . $auth->code . ' (' . $auth->message . ')', $auth->code == 200 ? 'ok' : 'error');
      }
    }
    return !empty($auth);
  }

  /**
   * Open an url on https using curl and return content

   *
   * @param string $url
   *   the url to open
   * @param array $post
   *   the post variables
   * @param bool $ssl
   *   if TRUE, use ssl
   * @param string $refer
   *   referer (optional)
   * @param mixed $usecookie
   *   if TRUE, cookie.txt will be used as default, or the usecookie value.
   * @param mixed $progress_function
   *   callback function with progress informations
   *
   * @return string
   */
  public function openUrl($url, $post = array(), $ssl = FALSE, $refer = '', $usecookie = FALSE, $progress_function = '') {

    if ($usecookie) {
      if (file_exists($usecookie)) {
        if (!is_writable($usecookie)) {
          return 'Can\'t write to ' . $usecookie . ' cookie file, ' . 'change file permission to 777 or remove read only for windows.';
        }
      }
      else {
        $usecookie = 'cookie.txt';
        if (!is_writable($usecookie)) {
          return 'Can\'t write to $usecookie cookie file, ' . 'change file permission to 777 or remove read only for windows.';
        }
      }
    }
    $ch = curl_init();

    curl_setopt($ch, CURLOPT_URL, $url);

    if ($ssl == TRUE) {
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
    }

    if (!empty($post)) {
      curl_setopt($ch, CURLOPT_POST, TRUE);
      curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
    }

    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Archibald/2.0 (@' . @$_SERVER['HTTP_HOST'] . ')');
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));

    if (!empty($progress_function) && defined('CURLOPT_PROGRESSFUNCTION')) {
      curl_setopt($ch, CURLOPT_NOPROGRESS, FALSE);
      curl_setopt($ch, CURLOPT_PROGRESSFUNCTION, $progress_function);
    }

    if (!empty($usecookie)) {
      curl_setopt($ch, CURLOPT_COOKIEJAR, $usecookie);
      curl_setopt($ch, CURLOPT_COOKIEFILE, $usecookie);
    }

    if (!empty($refer)) {
      curl_setopt($ch, CURLOPT_REFERER, $refer);
    }
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

    $result = curl_exec($ch);
    if (empty($result)) {
      $result = curl_exec($ch);
    }
    curl_close($ch);
    return $result;
  }

  /**
   * get open ssl ressource for private key
   *
   * @return resource
   */
  protected function getClientPrivateKeyFile() {
    $content_partner = $this->getContentPartnerCredential();

    if (empty($content_partner)) {
      return FALSE;
    }

    $private_key_file = $content_partner->key_file;
    return openssl_pkey_get_private(file_get_contents($private_key_file->uri), $content_partner->password);
  }

  /**
   * check if current content partner can access to central catalog
   *
   * @return bollean
   */
  public function contentPartnerCanCentral() {
    return ($this->getClientPrivateKeyFile() === FALSE) ? FALSE : TRUE;
  }

  /**
   * get api login credentials
   *
   * @return object
   *   username
   *   password
   *   key_fid
   *   key_file
   *   icon_fid
   *   icon_file
   *   file_server_type
   */
  protected function getContentPartnerCredential() {
    if (!empty($this->objectToPublish->lom_stats->content_partner)) {
      return (object) $this->objectToPublish->lom_stats->content_partner;
    }
    if (!empty($this->contentPartner)) {
      return $this->contentPartner;
    }

    return NULL;
  }

  /**
   * init content_partner by lom_id
   *
   * @param string $lom_id
   */
  public function loadContentPartnerByLomId($lom_id) {

    $stats = db_query('SELECT content_partner_id FROM {archibald_lom_stats} WHERE lom_id = :lom_id', array(':lom_id' => $lom_id));
    $stats_values = $stats->fetchAssoc();
    $this->contentPartner = (object) archibald_load_content_partner($stats_values['content_partner_id']);
  }

  /**
   * init content_partner by content_partner_id
   *
   * @param integer $content_partner_id
   */
  public function loadContentPartnerByContentPartnerId($content_partner_id) {
    $this->contentPartner = (object) archibald_load_content_partner($content_partner_id);
  }

  /**
   * Each sub class must implement this method.
   */
  public abstract function cron();
}
